@@ -87,7 +87,7 @@ as.phylo.phylog <- function(x, ...)
 as.hclust.phylo <- function(x, ...)
 {
     if (!is.ultrametric(x)) stop("the tree is not ultrametric")
-    if (!is.binary.tree(x)) stop("the tree is not binary")
+    if (!is.binary.phylo(x)) stop("the tree is not binary")
     if (!is.rooted(x)) stop("the tree is not rooted")
     n <- length(x$tip.label)
     x$node.label <- NULL # by Jinlong Zhang (2010-12-15)
